import * as React from "react";
import PropTypes from "prop-types";
import { alpha } from "@mui/material/styles";
import Box from "@mui/material/Box";
import Table from "@mui/material/Table";
import TableBody from "@mui/material/TableBody";
import TableCell from "@mui/material/TableCell";
import TableContainer from "@mui/material/TableContainer";
import TableHead from "@mui/material/TableHead";
import TablePagination from "@mui/material/TablePagination";
import TableRow from "@mui/material/TableRow";
import TableSortLabel from "@mui/material/TableSortLabel";
import Toolbar from "@mui/material/Toolbar";
import Typography from "@mui/material/Typography";
import IconButton from "@mui/material/IconButton";
import { useEffect, useState } from "react";
import InfoIcon from "@mui/icons-material/Info";
import axios from "axios";
import { makeStyles } from "@material-ui/core";
import { Link, Redirect } from "react-router-dom";
import Paper from "@mui/material/Paper";
import EnhancedTableToolbar from "./components/enchancedTableToolbar";
import { TextField } from "@mui/material";
import EnhancedTableHead from "./components/enchancedTableHead";
import { stableSort } from "./helpers/sorters";
import { getComparator } from "./helpers/comparators";
import Checkbox from "@mui/material/Checkbox";
import {
  deleteEmail,
  fetchEmails,
  fetchEmailsByGamerunID,
} from "../../helpers/emails";

const useStyles = makeStyles({
  ellipsis: {
    maxWidth: 200,
    whiteSpace: "nowrap",
    overflow: "hidden",
    textOverflow: "ellipsis",
  },
});

const rowsPerPage = 6;

const headCells = [
  {
    id: "email_id",
    numeric: true,
    disablePadding: true,
    label: "ID",
  },
  {
    id: "subject",
    numeric: false,
    disablePadding: false,
    label: "Subject",
  },
  {
    id: "body",
    numeric: false,
    disablePadding: false,
    label: "Body",
  },
  {
    id: "type",
    numeric: false,
    disablePadding: false,
    label: "Type",
  },
];

export default function EmailTable({ gamerunID }) {
  const classes = useStyles();
  const [data, setData] = useState([]);
  const [order, setOrder] = React.useState("asc");
  const [orderBy, setOrderBy] = React.useState("email_id");
  const [selected, setSelected] = React.useState([]);
  const [page, setPage] = React.useState(0);
  const [searchedData, setSearchedData] = useState([]);
  const [search, setSearch] = useState("");

  useEffect(() => {
    if (gamerunID) {
      fetchEmailsByGamerunID(gamerunID).then((emails) => setData(emails));
    } else {
      fetchEmails().then((emails) => {
        setData(emails);
      });
    }
  }, []);

  useEffect(() => {
    requestSearch(search);
  }, [data]);

  const requestSearch = (searchedVal) => {
    setSearch(searchedVal);
    setSelected([]);

    const filteredRows = data.filter((row) => {
      return (
        [row.email_id, row.subject, row.body, row.type.type].filter(
          (text) =>
            !!text &&
            text.toString().toLowerCase().includes(searchedVal.toLowerCase())
        ).length > 0
      );
    });

    setSearchedData(filteredRows);
  };

  const handleDeleteSelected = () => {
    let deletedEmails = [];

    selected.forEach((email_id) => {
      deleteEmail(email_id).then(deletedEmails.push(email_id));
    });

    const new_data = data.filter(
      (email) => deletedEmails.indexOf(email.email_id) === -1
    );

    setData(new_data);
    setSelected([]);
  };

  const handleRequestSort = (event, property) => {
    const isAsc = orderBy === property && order === "asc";
    setOrder(isAsc ? "desc" : "asc");
    setOrderBy(property);
  };

  const handleSelectAllClick = (event) => {
    if (event.target.checked) {
      const newSelecteds = searchedData.map((n) => n.email_id);
      setSelected(newSelecteds);
      return;
    }
    setSelected([]);
  };

  const handleClick = (event, id) => {
    const selectedIndex = selected.indexOf(id);
    let newSelected = [];

    if (selectedIndex === -1) {
      newSelected = newSelected.concat(selected, id);
    } else if (selectedIndex === 0) {
      newSelected = newSelected.concat(selected.slice(1));
    } else if (selectedIndex === selected.length - 1) {
      newSelected = newSelected.concat(selected.slice(0, -1));
    } else if (selectedIndex > 0) {
      newSelected = newSelected.concat(
        selected.slice(0, selectedIndex),
        selected.slice(selectedIndex + 1)
      );
    }

    setSelected(newSelected);
  };

  const handleChangePage = (event, newPage) => {
    setPage(newPage);
  };

  const isSelected = (name) => selected.indexOf(name) !== -1;

  // Avoid a layout jump when reaching the last page with empty rows.
  const emptyRows =
    page > 0
      ? Math.max(0, (1 + page) * rowsPerPage - searchedData.length)
      : rowsPerPage - searchedData.length;

  function Cells(row, id, isItemSelected) {
    return (
      <React.Fragment>
        <TableCell
          padding="checkbox"
          onClick={(event) => handleClick(event, row.email_id)}
        >
          <Checkbox
            color="primary"
            checked={isItemSelected}
            inputProps={{
              "aria-labelledby": id,
            }}
          />
        </TableCell>
        <TableCell
          component="th"
          id={row.email_id}
          padding="none"
          scope="row"
          align="right"
        >
          <Typography variant="inherit" className={classes.ellipsis}>
            {row.email_id}
          </Typography>
        </TableCell>
        <TableCell>
          <Typography variant="inherit" className={classes.ellipsis}>
            {row.subject}
          </Typography>
        </TableCell>
        <TableCell>
          <div
            style={{
              overflow: "hidden",
              textOverflow: "ellipsis",
              width: "11rem",
            }}
          >
            {row.body}
          </div>
        </TableCell>
        <TableCell>
          <Typography variant="inherit" className={classes.ellipsis}>
            {row.type.type}
          </Typography>
        </TableCell>
        <TableCell padding="checkbox">
          <Link to={`emails/${row.email_id}`}>
            <IconButton color="primary">
              <InfoIcon />
            </IconButton>
          </Link>
        </TableCell>
      </React.Fragment>
    );
  }

  return (
    <Box
      justifyContent="center"
      alignContent="center"
      sx={{ display: "flex", width: "100%" }}
    >
      <Paper sx={{ minWidth: "350px", width: "95%", mt: 5 }}>
        <EnhancedTableToolbar
          numSelected={selected.length}
          onDelete={handleDeleteSelected}
          name="Emails"
        />
        <TextField
          style={{ ml: 5 }}
          value={search}
          label="search"
          size="small"
          onChange={(val) => requestSearch(val.target.value)}
        />
        <TableContainer>
          <Table aria-labelledby="tableTitle" size="medium">
            <EnhancedTableHead
              numSelected={selected.length}
              order={order}
              orderBy={orderBy}
              onSelectAllClick={handleSelectAllClick}
              onRequestSort={handleRequestSort}
              rowCount={data.length}
              headCells={headCells}
              iconCell={true}
            />
            <TableBody>
              {stableSort(searchedData, getComparator(order, orderBy))
                .slice(page * rowsPerPage, page * rowsPerPage + rowsPerPage)
                .map((row, index) => {
                  const isItemSelected = isSelected(row.email_id);
                  const id = `enhanced-table-checkbox-${index}`;

                  return (
                    <TableRow
                      hover
                      role="checkbox"
                      aria-checked={isItemSelected}
                      tabIndex={-1}
                      key={row.email_id}
                      selected={isItemSelected}
                    >
                      {Cells(row, id, isItemSelected)}
                    </TableRow>
                  );
                })}
              {emptyRows > 0 && (
                <TableRow
                  style={{
                    height: 53 * emptyRows,
                  }}
                >
                  <TableCell colSpan={6} />
                </TableRow>
              )}
            </TableBody>
          </Table>
        </TableContainer>
        <TablePagination
          rowsPerPageOptions={[]}
          labelRowsPerPage=""
          component="div"
          count={data.length}
          rowsPerPage={rowsPerPage}
          page={page}
          onPageChange={handleChangePage}
        />
      </Paper>
    </Box>
  );
}
