import * as React from "react";
import { useEffect, useState } from "react";
import Box from "@mui/material/Box";
import Table from "@mui/material/Table";
import TableBody from "@mui/material/TableBody";
import TableCell from "@mui/material/TableCell";
import TableContainer from "@mui/material/TableContainer";
import TablePagination from "@mui/material/TablePagination";
import TableRow from "@mui/material/TableRow";
import Typography from "@mui/material/Typography";
import Paper from "@mui/material/Paper";
import Checkbox from "@mui/material/Checkbox";
import IconButton from "@mui/material/IconButton";
import InfoIcon from "@mui/icons-material/Info";
import { makeStyles } from "@material-ui/core";
import { TextField } from "@mui/material";
import { Link } from "react-router-dom";
import EnhancedTableToolbar from "./components/enchancedTableToolbar";
import EnhancedTableHead from "./components/enchancedTableHead";
import { stableSort } from "./helpers/sorters";
import { getComparator } from "./helpers/comparators";
import { deleteGamerun, fetchGameruns } from "../../helpers/gameruns";

const rowsPerPage = 6;

const useStyles = makeStyles({
  ellipsis: {
    maxWidth: 200,
    whiteSpace: "nowrap",
    overflow: "hidden",
    textOverflow: "ellipsis",
  },
});

const headCells = [
  {
    id: "gamerun_id",
    numeric: true,
    disablePadding: true,
    label: "ID",
  },
  {
    id: "player_name",
    numeric: false,
    disablePadding: false,
    label: "Username",
  },
  {
    id: "start_time",
    numeric: false,
    disablePadding: false,
    label: "Start time",
  },
  {
    id: "end_time",
    numeric: false,
    disablePadding: false,
    label: "End time",
  },
];

export default function GamerunTable() {
  const classes = useStyles();
  const [data, setData] = useState([]);
  const [order, setOrder] = React.useState("asc");
  const [orderBy, setOrderBy] = React.useState("gamerun_id");
  const [selected, setSelected] = React.useState([]);
  const [page, setPage] = React.useState(0);
  const [searchedData, setSearchedData] = useState([]);
  const [search, setSearch] = useState("");

  useEffect(() => {
    fetchGameruns().then((res) => setData(res));
  }, []);

  useEffect(() => {
    requestSearch(search);
  }, [data]);

  const requestSearch = (searchedVal) => {
    setSearch(searchedVal);
    setSelected([]);

    const filteredRows = data.filter((row) => {
      return (
        [row.gamerun_id, row.player_name, row.start_time, row.end_time].filter(
          (text) =>
            !!text &&
            text.toString().toLowerCase().includes(searchedVal.toLowerCase())
        ).length > 0
      );
    });
    console.log(filteredRows);
    setSearchedData(filteredRows);
  };

  const handleDeleteSelected = () => {
    let deletedGameruns = [];

    selected.forEach((gamerun_id) => {
      deleteGamerun(gamerun_id).then(deletedGameruns.push(gamerun_id));
    });

    const new_data = data.filter(
      (gamerun) => deletedGameruns.indexOf(gamerun.gamerun_id) === -1
    );

    setData(new_data);
    setSelected([]);
  };

  const handleRequestSort = (event, property) => {
    const isAsc = orderBy === property && order === "asc";
    setOrder(isAsc ? "desc" : "asc");
    setOrderBy(property);
  };

  const handleSelectAllClick = (event) => {
    if (event.target.checked) {
      const newSelecteds = data.map((n) => n.gamerun_id);
      setSelected(newSelecteds);
      return;
    }
    setSelected([]);
  };

  const handleClick = (event, id) => {
    const selectedIndex = selected.indexOf(id);
    let newSelected = [];

    if (selectedIndex === -1) {
      newSelected = newSelected.concat(selected, id);
    } else if (selectedIndex === 0) {
      newSelected = newSelected.concat(selected.slice(1));
    } else if (selectedIndex === selected.length - 1) {
      newSelected = newSelected.concat(selected.slice(0, -1));
    } else if (selectedIndex > 0) {
      newSelected = newSelected.concat(
        selected.slice(0, selectedIndex),
        selected.slice(selectedIndex + 1)
      );
    }

    setSelected(newSelected);
  };

  const handleChangePage = (event, newPage) => {
    setPage(newPage);
  };

  const isSelected = (name) => selected.indexOf(name) !== -1;

  const emptyRows =
    page > 0
      ? Math.max(0, (1 + page) * rowsPerPage - searchedData.length)
      : rowsPerPage - searchedData.length;

  const TableCells = (row, id, isItemSelected) => (
    <React.Fragment>
      <TableCell
        padding="checkbox"
        onClick={(event) => handleClick(event, row.gamerun_id)}
      >
        <Checkbox
          color="primary"
          checked={isItemSelected}
          inputProps={{
            "aria-labelledby": id,
          }}
        />
      </TableCell>
      <TableCell
        component="th"
        id={id}
        padding="none"
        scope="row"
        align="right"
      >
        <Typography variant="inherit" className={classes.ellipsis}>
          {row.gamerun_id}
        </Typography>
      </TableCell>

      <TableCell>
        <div
          style={{
            overflow: "hidden",
            textOverflow: "ellipsis",
            width: "11rem",
          }}
        >
          {row.player_name}
        </div>
      </TableCell>
      <TableCell>
        <Typography variant="inherit" className={classes.ellipsis}>
          {row.start_time}
        </Typography>
      </TableCell>
      <TableCell>{row.end_time}</TableCell>
      <TableCell padding="checkbox">
        <Link to={`gameruns/${row.gamerun_id}`}>
          <IconButton color="primary">
            <InfoIcon />
          </IconButton>
        </Link>
      </TableCell>
    </React.Fragment>
  );

  return (
    <Box
      justifyContent="center"
      alignContent="center"
      sx={{ display: "flex", width: "100%" }}
    >
      <Paper sx={{ minWidth: "350px", width: "100%" }}>
        <EnhancedTableToolbar
          numSelected={selected.length}
          onDelete={handleDeleteSelected}
          name="Gameruns"
        />
        <TextField
          style={{ ml: 5 }}
          value={search}
          label="search"
          size="small"
          onChange={(val) => requestSearch(val.target.value)}
        />
        <TableContainer>
          <Table aria-labelledby="tableTitle" size="medium">
            <EnhancedTableHead
              numSelected={selected.length}
              order={order}
              orderBy={orderBy}
              onSelectAllClick={handleSelectAllClick}
              onRequestSort={handleRequestSort}
              rowCount={data.length}
              headCells={headCells}
              iconCell={true}
            />

            <TableBody>
              {stableSort(searchedData, getComparator(order, orderBy))
                .slice(page * rowsPerPage, page * rowsPerPage + rowsPerPage)
                .map((row, index) => {
                  const isItemSelected = isSelected(row.gamerun_id);
                  const id = `enhanced-table-checkbox-${index}`;

                  return (
                    <TableRow
                      hover
                      role="checkbox"
                      aria-checked={isItemSelected}
                      tabIndex={-1}
                      key={row.gamerun_id}
                      selected={isItemSelected}
                    >
                      {TableCells(row, id, isItemSelected)}
                    </TableRow>
                  );
                })}
              {emptyRows > 0 && (
                <TableRow
                  style={{
                    height: 53 * emptyRows,
                  }}
                >
                  <TableCell colSpan={6} />
                </TableRow>
              )}
            </TableBody>
          </Table>
        </TableContainer>
        <TablePagination
          rowsPerPageOptions={[]}
          labelRowsPerPage=""
          component="div"
          count={data.length}
          rowsPerPage={rowsPerPage}
          page={page}
          onPageChange={handleChangePage}
        />
      </Paper>
    </Box>
  );
}
