import * as React from "react";
import { useEffect, useState } from "react";
import Box from "@mui/material/Box";
import Table from "@mui/material/Table";
import TableBody from "@mui/material/TableBody";
import TableCell from "@mui/material/TableCell";
import TableContainer from "@mui/material/TableContainer";
import TablePagination from "@mui/material/TablePagination";
import TableRow from "@mui/material/TableRow";
import Paper from "@mui/material/Paper";
import Checkbox from "@mui/material/Checkbox";
import IconButton from "@mui/material/IconButton";
import axios from "axios";
import { TextField } from "@mui/material";
import AddCircleIcon from "@mui/icons-material/AddCircle";
import EnhancedTableHead from "./components/enchancedTableHead";
import EnhancedTableToolbar from "./components/enchancedTableToolbar";
import { stableSort } from "./helpers/sorters";
import { getComparator } from "./helpers/comparators";
import { deleteSign, postSign } from "../../helpers/signs";

const rowsPerPage = 5;

const headCells = [
  {
    id: "sign_id",
    numeric: true,
    disablePadding: true,
    label: "ID",
  },
  {
    id: "sign",
    numeric: false,
    disablePadding: false,
    label: "Sign",
  },
];

export default function SignTable({ signs = [], handleDelete, allowCreate }) {
  const [order, setOrder] = React.useState("asc");
  const [orderBy, setOrderBy] = React.useState("id");
  const [selected, setSelected] = React.useState([]);
  const [page, setPage] = React.useState(0);
  const [data, setData] = useState(signs);
  const [searchedData, setSearchedData] = useState(signs);
  const [search, setSearch] = useState("");
  const [signName, setSignName] = useState("");

  useEffect(() => {
    setData(signs);
  }, [signs]);

  useEffect(() => {
    requestSearch(search);
  }, [data]);

  const requestSearch = (searchedVal) => {
    setSelected([]);
    setSearch(searchedVal);
    const filteredRows = data.filter((row) => {
      return (
        [row.sign_id, row.text].filter((text) =>
          text.toString().toLowerCase().includes(searchedVal.toLowerCase())
        ).length > 0
      );
    });
    setSearchedData(filteredRows);
  };

  function handleDeleteSelected() {
    if (handleDelete !== undefined) {
      handleDelete(selected);
      requestSearch(search);
      return;
    }

    let deletedSigns = [];

    selected.forEach((sign_id) => {
      deleteSign(sign_id).then(deletedSigns.push(sign_id));
    });

    const new_data = data.filter(
      (sign) => deletedSigns.indexOf(sign.sign_id) === -1
    );
    setSelected([]);
    setData(new_data);
  }
  const handleRequestSort = (event, property) => {
    const isAsc = orderBy === property && order === "asc";
    setOrder(isAsc ? "desc" : "asc");
    setOrderBy(property);
  };

  const handleSelectAllClick = (event) => {
    if (event.target.checked) {
      const newSelecteds = searchedData.map((n) => n.sign_id);
      setSelected(newSelecteds);
      return;
    }
    setSelected([]);
  };

  const handleClick = (event, id) => {
    const selectedIndex = selected.indexOf(id);
    let newSelected = [];

    if (selectedIndex === -1) {
      newSelected = newSelected.concat(selected, id);
    } else if (selectedIndex === 0) {
      newSelected = newSelected.concat(selected.slice(1));
    } else if (selectedIndex === selected.length - 1) {
      newSelected = newSelected.concat(selected.slice(0, -1));
    } else if (selectedIndex > 0) {
      newSelected = newSelected.concat(
        selected.slice(0, selectedIndex),
        selected.slice(selectedIndex + 1)
      );
    }

    setSelected(newSelected);
  };

  const handleChangePage = (event, newPage) => {
    setPage(newPage);
  };

  const isSelected = (id) => selected.indexOf(id) !== -1;

  // Avoid a layout jump when reaching the last page with empty rows.
  const emptyRows =
    page > 0
      ? Math.max(0, (1 + page) * rowsPerPage - searchedData.length)
      : rowsPerPage - searchedData.length;

  function handleSignNameChange(e) {
    setSignName(e.target.value);
  }

  function handleCreateSign() {
    if (signName.trim().length > 0) {
      postSign({ text: signName }).then((res) => {
        setData([...data, res.data]);
      });
    }
    setSignName("");
  }

  const cells = (row, id, isItemSelected) => {
    return (
      <React.Fragment>
        <TableCell padding="checkbox">
          <Checkbox
            color="primary"
            checked={isItemSelected}
            inputProps={{
              "aria-labelledby": id,
            }}
          />
        </TableCell>
        <TableCell
          component="th"
          id={id}
          padding="none"
          scope="row"
          align="right"
        >
          {row.sign_id}
        </TableCell>
        <TableCell>
          <div
            style={{
              overflow: "hidden",
              textOverflow: "ellipsis",
              width: "11rem",
            }}
          >
            {row.text}
          </div>
        </TableCell>
      </React.Fragment>
    );
  };

  return (
    <Box
      justifyContent="center"
      alignContent="center"
      sx={{ display: "flex", width: "100%" }}
    >
      <Paper sx={{ width: "90%" }}>
        <EnhancedTableToolbar
          numSelected={selected.length}
          onDelete={handleDeleteSelected}
          name="Signs"
        />
        <TextField
          style={{ ml: 5 }}
          value={search}
          label="search"
          size="small"
          onChange={(val) => requestSearch(val.target.value)}
        />

        <TableContainer>
          <Table sx={{}} aria-labelledby="tableTitle" size="medium">
            <EnhancedTableHead
              numSelected={selected.length}
              order={order}
              orderBy={orderBy}
              onSelectAllClick={handleSelectAllClick}
              onRequestSort={handleRequestSort}
              rowCount={selected.length}
              headCells={headCells}
            />
            <TableBody>
              {stableSort(searchedData, getComparator(order, orderBy))
                .slice(page * rowsPerPage, page * rowsPerPage + rowsPerPage)
                .map((row, index) => {
                  const isItemSelected = isSelected(row.sign_id);
                  const id = `enhanced-table-checkbox-${index}`;

                  return (
                    <TableRow
                      hover
                      onClick={(event) => handleClick(event, row.sign_id)}
                      role="checkbox"
                      aria-checked={isItemSelected}
                      tabIndex={-1}
                      key={row.sign_id}
                      selected={isItemSelected}
                    >
                      {cells(row, id, isItemSelected)}
                    </TableRow>
                  );
                })}
              {emptyRows > 0 && (
                <TableRow
                  style={{
                    height: 53 * emptyRows,
                  }}
                >
                  <TableCell />
                  <TableCell />
                  <TableCell />
                </TableRow>
              )}
            </TableBody>
          </Table>
        </TableContainer>
        <Box style={{ display: "flex", paddingLeft: 50, paddingRight: 50 }}>
          {allowCreate && (
            <TextField
              fullWidth
              size="small"
              style={{ px: 5 }}
              value={signName}
              label="New sign"
              onChange={handleSignNameChange}
              InputProps={{
                endAdornment: (
                  <IconButton onClick={handleCreateSign}>
                    <AddCircleIcon color="primary" />
                  </IconButton>
                ),
              }}
            />
          )}
        </Box>
        <TablePagination
          rowsPerPageOptions={[]}
          labelRowsPerPage=""
          component="div"
          count={searchedData.length}
          rowsPerPage={rowsPerPage}
          page={page}
          onPageChange={handleChangePage}
        />
      </Paper>
    </Box>
  );
}
